CREATE TABLE Pais
(
  Id_Pais   INTEGER NOT NULL,
  Nombre    VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Pais PRIMARY KEY (Id_Pais)
);
CREATE TABLE Estado
(
  Id_Estado INTEGER NOT NULL,
  Nombre    VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Estado PRIMARY KEY (Id_Estado)
);
CREATE TABLE Region
(
  Id_Region INTEGER NOT NULL,
  Nombre    VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Region PRIMARY KEY (Id_Region)
);
CREATE TABLE Partido
(
  Id_Partido    INTEGER NOT NULL,
  Nombre        VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Partido PRIMARY KEY (Id_Partido)
);
CREATE TABLE Estado_Gasto
(
  Id_Estado_Gasto   INTEGER NOT NULL,
  Descripcion       VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Estado_Gasto PRIMARY KEY (Id_Estado_Gasto)
);
CREATE TABLE Categoria_Gasto
(
  Id_Categoria  INTEGER NOT NULL,
  Tipo_Gasto    VARCHAR2(1) NOT NULL, 
  Descripcion   VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Categoria_Gasto PRIMARY KEY (Id_Categoria),
  CONSTRAINT CK_Categoria_Gasto CHECK (Tipo_Gasto IN('P', 'C'))
);
CREATE TABLE Forma_Pago
(
  Id_Pago      INTEGER NOT NULL,
  Descripcion  VARCHAR2(200) NOT NULL, 
  CONSTRAINT PK_Forma_Pago PRIMARY KEY (Id_Pago)
);
CREATE TABLE Camara
(
  Id_Camara         INTEGER NOT NULL,
  Nombre            VARCHAR2(200) NOT NULL,
  No_Parlamentarios INTEGER,
  Id_Pais           INTEGER,
  Id_Estado         INTEGER,
  Id_Region         INTEGER,
  Direccion         VARCHAR2(200),
  Tlfno             NUMBER(12),
  Web               VARCHAR2(200),
  CONSTRAINT PK_Camara PRIMARY KEY (Id_Camara),
  CONSTRAINT FK_Camara_Pais  FOREIGN KEY (Id_Pais) REFERENCES Pais (Id_Pais),
  CONSTRAINT FK_Camara_Estado  FOREIGN KEY (Id_Estado) REFERENCES Estado (Id_Estado),
  CONSTRAINT FK_Camara_Region  FOREIGN KEY (Id_Region) REFERENCES Region (Id_Region)
);

CREATE TABLE Parlamentario
(
  NIF               VARCHAR2(9) not null,
  Nombre_Apellidos  VARCHAR2(200) not null,
  Id_Partido        INTEGER not null,
  CONSTRAINT PK_Parlamentario PRIMARY KEY (NIF),
  CONSTRAINT FK_Parlamentario_Partido  FOREIGN KEY (Id_Partido) REFERENCES Partido (Id_Partido)
);

CREATE TABLE Parlamentario_Camara
(
   Id_Parlamentario         INTEGER NOT NULL,
   Id_Camara                INTEGER NOT NULL,
   NIF                      VARCHAR2(9) NOT NULL,
   Fecha_Alta               DATE NOT NULL,
   Fecha_Baja               DATE NOT NULL,
   CONSTRAINT PK_Parlamentario_Camara PRIMARY KEY (Id_Parlamentario),
   CONSTRAINT FK_Parlamentario_Camara_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara),
   CONSTRAINT FK_Parlamentario_Camara_Parlam  FOREIGN KEY (NIF) REFERENCES Parlamentario (NIF)
);

CREATE TABLE Gasto
(
   Id_Gasto          INTEGER NOT NULL,
   Id_Parlamentario  INTEGER,
   Id_Camara         INTEGER NOT NULL,
   Id_Partido        INTEGER,
   Fecha             DATE NOT NULL,
   Importe           NUMBER(12,2) NOT NULL,
   IVA               INTEGER NOT NULL,
   NIF_Prov          VARCHAR2(9),
   Id_Categoria      INTEGER NOT NULL,
   Descripcion       VARCHAR2(200) NOT NULL,
   Justificacion     VARCHAR2(200) NOT NULL,
   URL               VARCHAR2(200),
   Id_Estado_Gasto   INTEGER NOT NULL,
   Id_Pago           INTEGER NOT NULL,
   CONSTRAINT PK_Gasto PRIMARY KEY (Id_Gasto),
   CONSTRAINT FK_Gasto_Parlamentario_Camara  FOREIGN KEY (Id_Parlamentario) REFERENCES Parlamentario_Camara (Id_Parlamentario),
   CONSTRAINT FK_Gasto_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara),
   CONSTRAINT FK_Gasto_Partido  FOREIGN KEY (Id_Partido) REFERENCES Partido (Id_Partido),
   CONSTRAINT FK_Gasto_Categoria  FOREIGN KEY (Id_Partido) REFERENCES Partido (Id_Partido),
   CONSTRAINT FK_Gasto_Estado_Gasto  FOREIGN KEY (Id_Estado_Gasto) REFERENCES Estado_Gasto (Id_Estado_Gasto),
   CONSTRAINT FK_Gasto_Forma_Pago  FOREIGN KEY (Id_Pago) REFERENCES Forma_Pago (Id_Pago)
);

CREATE TABLE Estadistica1
(
   Id_Camara    INTEGER NOT NULL,
   Gasto        NUMBER(12,2) NOT NULL,
   CONSTRAINT PK_Est1 PRIMARY KEY (Id_Camara),
   CONSTRAINT FK_Est1_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara)
);

CREATE TABLE Estadistica2
(
   Anyo         INTEGER NOT NULL,
   Id_Camara    INTEGER NOT NULL,
   Diferencia   NUMBER(12,2) NOT NULL,
   CONSTRAINT PK_Est2 PRIMARY KEY (Anyo,Id_Camara),
   CONSTRAINT FK_Est2_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara)
);

CREATE TABLE Estadistica3
(
   Anyo         INTEGER NOT NULL,
   Id_Camara    INTEGER NOT NULL,
   CONSTRAINT PK_Est3 PRIMARY KEY (Anyo),
   CONSTRAINT FK_Est3_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara)
);

CREATE TABLE Estadistica4
(
   NIF      VARCHAR2(9) NOT NULL,
   Gasto    NUMBER(12,2) NOT NULL,
   CONSTRAINT PK_Est4 PRIMARY KEY (NIF),
   CONSTRAINT FK_Est4_Parlamentario  FOREIGN KEY (NIF) REFERENCES Parlamentario (NIF)
);

CREATE TABLE Estadistica5
(
   Anyo     INTEGER NOT NULL,
   Media    NUMBER(12,2) NOT NULL,
   CONSTRAINT PK_Est5 PRIMARY KEY (Anyo)
);

CREATE TABLE Estadistica6
(
   Anyo     INTEGER NOT NULL,
   NIF_Mas_Gasto      VARCHAR2(9) NOT NULL,
   NIF_Menos_Gasto      VARCHAR2(9) NOT NULL,
   CONSTRAINT PK_Est6 PRIMARY KEY (Anyo),
   CONSTRAINT FK_Est6_Parlamentario  FOREIGN KEY (NIF_Mas_Gasto) REFERENCES Parlamentario (NIF),
   CONSTRAINT FK_Est6_Parlamentario_2  FOREIGN KEY (NIF_Menos_Gasto) REFERENCES Parlamentario (NIF)
);

CREATE TABLE Estadistica7
(
   Anyo         INTEGER NOT NULL,
   Id_Camara    INTEGER NOT NULL,
   Nombre       VARCHAR2(200) NOT NULL,
   Media_Gasto  NUMBER(12,2),
   CONSTRAINT PK_Est7 PRIMARY KEY (Anyo),
   CONSTRAINT FK_Est7_Camara  FOREIGN KEY (Id_Camara) REFERENCES Camara (Id_Camara)
);

CREATE TABLE Estadistica8
(
   Anyo     INTEGER NOT NULL,
   Gasto    NUMBER(12,2),
   CONSTRAINT PK_Est8 PRIMARY KEY (Anyo)
);

CREATE TABLE Log
(
   Id_Log               INTEGER NOT NULL,
   Procedimiento        VARCHAR2(200) NOT NULL,
   Parametros_Entrada   VARCHAR2(200) NOT NULL,
   Parametros_Salida    VARCHAR2(200) NOT NULL,
   Fecha                DATE NOT NULL,
   CONSTRAINT PK_Log PRIMARY KEY (Id_Log)
);
