CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_LOG (
   p_procedimiento        IN TABLA_LOG.procedimiento%TYPE,
   p_parametros_entrada   IN TABLA_LOG.parametros_entrada%TYPE,
   p_parametros_salida    IN TABLA_LOG.parametros_salida%TYPE,
   RSP                    OUT VARCHAR2
)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_LOG
   Obejtivo:               aadir una entrada a la tabla de log
                   
   Parmetros de entrada:  p_procedimento, p_parametros_entrada, p_parametros_salida            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_campo_nulo         VARCHAR2 (20);
E_CAMPO_NULO         EXCEPTION;

BEGIN
   v_parametros_entrada:= 'procedimiento: '||p_procedimiento||
                 ', parametros_entrada: '||p_parametros_entrada||
                 ', parametros_salida:  '||p_parametros_salida;

  -- Comprueba que ninguno de los parmetros de entrada sea nulo
   IF (p_procedimiento IS NULL) THEN 
     v_campo_nulo:= 'procedimiento';     
     RAISE E_CAMPO_NULO;
   ELSIF (p_parametros_entrada IS NULL) THEN 
     v_campo_nulo:= 'parametros_entrada';
     RAISE E_CAMPO_NULO;     
   ELSIF (p_parametros_salida IS NULL) THEN 
     v_campo_nulo:= 'parametros_salida';
     RAISE E_CAMPO_NULO;     
   END IF;
 

  -- inserta una nueva entrada en la tabla de log
   INSERT INTO TABLA_LOG (procedimiento, parametros_entrada, parametros_salida, fecha)
   VALUES (p_procedimiento, p_parametros_entrada, p_parametros_salida, sysdate);
   
  -- terminado ok
   RSP:= 'OK';

   COMMIT;
EXCEPTION
   WHEN E_CAMPO_NULO THEN
        RSP:= 'ERROR LOG: El campo ' || v_campo_nulo || ' no puede ser nulo'; 
        INSERT INTO TABLA_LOG (procedimiento, parametros_entrada, parametros_salida, fecha)
        VALUES ('SP_ALTA_LOG', v_parametros_entrada, 'RSP: '||RSP, sysdate);
        
   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR LOG: ' || v_err_msg;
        INSERT INTO TABLA_LOG (procedimiento, parametros_entrada, parametros_salida, fecha)
        VALUES ('SP_ALTA_LOG', v_parametros_entrada, 'RSP: '||RSP, sysdate);
              
END SP_ALTA_LOG;
/
