CREATE OR REPLACE PROCEDURE TFC.SP_EST_1 (
   p_id_camara         IN ESTADISTICA1.id_camara%TYPE,
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_1
   Obejtivo:               actualiza la tabla Estadsitca1
                   
   Parmetros de entrada:  p_id_camara            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_total_gasto        NUMBER(14,2);
v_integridad         INTEGER; 
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
    v_parametros_entrada:= 'id_camara: '|| to_char(p_id_camara);
   
   -- Comprueba si el parmetro de entrada sea nulo
    IF (p_id_camara IS NULL) THEN 
      RSP:='El campo id_camara no puede ser nulo';     
      RAISE E_CONOCIDA;
    END IF;
 
    -- Comprueba si la cmara existe
    select count(*) into v_integridad from camara where id_camara = p_id_camara;
    IF (v_integridad=0) THEN
       RSP:='No existe ninguna cmara con ese id';     
       RAISE E_CONOCIDA;
    END IF;
   
    -- Calcula el importe del gasto de la cmara
    select sum(importe) into v_total_gasto from gasto where id_camara = p_id_camara and to_number(to_char(fecha,'yyyy')) between to_number(to_char(sysdate,'yyyy'))-4 and to_number(to_char(sysdate,'yyyy'))-1; 
    
    -- Comprueba si en la tabla estadstica1 ya exsisten datos para esa cmara
    select count(*) into v_existe_registro from estadistica1 where id_camara = p_id_camara;
   
    IF (v_existe_registro=0 and v_total_gasto is not null) THEN
       insert into estadistica1 values (p_id_camara,v_total_gasto);
    ELSIF (v_existe_registro=1 and v_total_gasto is not null) THEN
       update estadistica1 set gasto=v_total_gasto where id_camara=p_id_camara;
    ELSIF (v_existe_registro=1 and v_total_gasto is null) THEN
       delete from estadistica1 where id_camara=p_id_camara;
    END IF;
   
   -- terminado ok
    RSP:= 'OK';

   -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_1', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_1: ' || RSP;
        SP_ALTA_LOG('SP_EST_1', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_1: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_1', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_1;
/
