CREATE OR REPLACE PROCEDURE TFC.SP_EST_2 (
   p_anyo              IN ESTADISTICA2.anyo%TYPE,
   p_id_camara         IN ESTADISTICA2.id_camara%TYPE,   
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_2
   Obejtivo:               actualiza la tabla Estadsitca2
                   
   Parmetros de entrada:  p_anyo, p_id_camara            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_diferencia         NUMBER(14,2); 
v_integridad         INTEGER;
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION; 

BEGIN
   v_parametros_entrada:='id_camara: '|| to_char(p_id_camara)
                      || 'anyo: '     || to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_id_camara IS NULL) THEN 
     RSP:='El campo id_camara no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Comprueba si la cmara existe
   select count(*) into v_integridad from camara where id_camara = p_id_camara;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Calcula la diferencia de gastos entre los parmentarios
   select max(importe)-min(importe) into v_diferencia from (select sum(importe) importe from gasto where id_camara = p_id_camara and to_number(to_char(fecha,'yyyy')) = p_anyo group by id_parlamentario);
     
   -- Comprueba si en la tabla estadstica 2 ya existen datos para la camara y el ao
   select count(*) into v_existe_registro from estadistica2 where id_camara = p_id_camara and anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_diferencia IS NOT NULL) THEN
     insert into estadistica2 values (p_anyo, p_id_camara,v_diferencia);
   ELSIF (v_existe_registro=1 AND v_diferencia IS NOT NULL) THEN
     update estadistica2 set estadistica2.diferencia=diferencia where id_camara=p_id_camara and anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_diferencia IS NULL) THEN
     delete from estadistica2 where id_camara=p_id_camara and anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_2', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_2: ' || RSP;
        SP_ALTA_LOG('SP_EST_2', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR SP_EST_2: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_2', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_2;
/
