CREATE OR REPLACE PROCEDURE TFC.SP_EST_3 (
   p_anyo              IN ESTADISTICA3.anyo%TYPE,   
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_3
   Obejtivo:               actualiza la tabla Estadsitca3
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
v_id_camara          INTEGER; 
v_imp_max            NUMBER(14,2);
E_CONOCIDA           EXCEPTION;  

BEGIN
   v_parametros_entrada:= 'anyo: '|| to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Calcula la cmara con ms gastos para el ao dado
   
   select max(sum(importe)) into v_imp_max from gasto where to_number(to_char(fecha,'yyyy'))=p_anyo group by id_camara;
   
   select id_camara into v_id_camara from gasto where to_number(to_char(fecha,'yyyy'))=p_anyo having sum(importe)=v_imp_max group by id_camara;
     
   -- Comprueba si en la tabla estadstica 3 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA3 where anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_id_camara IS NOT NULL) THEN
     insert into ESTADISTICA3 values (p_anyo, v_id_camara);
   ELSIF (v_existe_registro=1 AND v_id_camara IS NOT NULL) THEN
     update ESTADISTICA3 set id_camara=v_id_camara where anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_id_camara IS NULL) THEN
     delete from ESTADISTICA3 where anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_3', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_3: ' || RSP;
        SP_ALTA_LOG('SP_EST_3', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_3: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_3', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_3;
/
