CREATE OR REPLACE PROCEDURE TFC.SP_EST_4 (
   p_NIF              IN ESTADISTICA4.NIF%TYPE,
   RSP                OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_4
   Obejtivo:               actualiza la tabla Estadsitca4
                   
   Parmetros de entrada:  p_NIF            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_importe            NUMBER(14,2);
v_existe_registro    INTEGER;
E_CONOCIDA           EXCEPTION;  

BEGIN
   v_parametros_entrada:= 'p_NIF: '|| p_NIF;
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_NIF IS NULL) THEN 
     RSP:='El campo NIF no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Comprueba si el parlamentario existe
   select count(*) into v_integridad from parlamentario where NIF = p_NIF;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ningn parlamentario con ese NIF';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Calcula el gasto total del parlamentario en los ltimos 4 aos
   
   select sum(importe) into v_importe from gasto a, parlamentario_camara b 
   where a.id_parlamentario=b.id_parlamentario and b.NIF=p_NIF and to_number(to_char(fecha,'yyyy')) between to_number(to_char(sysdate,'yyyy'))-4 and to_number(to_char(sysdate,'yyyy'))-1;
     
   -- Comprueba si en la tabla estadstica 3 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA4 where NIF = p_NIF;
   
   IF (v_existe_registro=0 AND v_importe IS NOT NULL) THEN
     insert into ESTADISTICA4 values (p_NIF, v_importe);
   ELSIF (v_existe_registro=1 AND v_importe IS NOT NULL) THEN
     update ESTADISTICA4 set GASTO=v_importe where NIF = p_NIF;
   ELSIF (v_existe_registro=1 AND v_importe IS NULL) THEN
     delete from ESTADISTICA4 where NIF = p_NIF;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_4', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_4: ' || RSP;
        SP_ALTA_LOG('SP_EST_4', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_4: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_4', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_4;
/
