CREATE OR REPLACE PROCEDURE TFC.SP_EST_5 (
   p_anyo              IN ESTADISTICA5.anyo%TYPE,
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_5
   Obejtivo:               actualiza la tabla Estadsitca5
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
v_num_parlamentarios INTEGER; 
v_total_gasto        NUMBER (14,2);
v_anyo_actual        INTEGER;
E_CONOCIDA           EXCEPTION;   

BEGIN
   v_parametros_entrada:= 'anyo: '|| to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Calcula el importe total del gasto de los parlamentarios y el nmero de parlamentarios en el ao indicado
   select sum(importe) into v_total_gasto from gasto where to_number(to_char(fecha,'yyyy'))=p_anyo and id_parlamentario is not null;
   
   select to_number(to_char(sysdate,'yyyy')) into v_anyo_actual from dual;
   
   IF (p_anyo=v_anyo_actual) THEN
      select count(distinct NIF) into v_num_parlamentarios from parlamentario_camara where to_number(to_char(fecha_baja,'yyyy'))=p_anyo OR fecha_baja IS NULL;
   ELSE
      select count(distinct NIF) into v_num_parlamentarios from parlamentario_camara where to_number(to_char(fecha_alta,'yyyy'))<=p_anyo AND to_number(to_char(fecha_baja,'yyyy'))>=p_anyo;
   END IF;  
   -- Comprueba si en la tabla estadstica 5 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA5 where anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_total_gasto IS NOT NULL) THEN
     insert into ESTADISTICA5 values (p_anyo, v_total_gasto/v_num_parlamentarios);
   ELSIF (v_existe_registro=1 AND v_total_gasto IS NOT NULL) THEN
     update ESTADISTICA5 set media=v_total_gasto/v_num_parlamentarios where anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_total_gasto IS NULL) THEN
     delete from ESTADISTICA5 where anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_5', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_5: ' || RSP;
        SP_ALTA_LOG('SP_EST_5', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'SP_EST_5: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_5', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_5;
/
