CREATE OR REPLACE PROCEDURE TFC.SP_EST_6 (
   p_anyo              IN ESTADISTICA6.anyo%TYPE,
   RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_6
   Obejtivo:               actualiza la tabla Estadsitca6
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
v_maximo             NUMBER(14,2); 
v_minimo             NUMBER(14,2);
v_NIF_maximo         VARCHAR2(9);
v_NIF_minimo         VARCHAR2(9);
E_CONOCIDA           EXCEPTION;   

BEGIN
   v_parametros_entrada:= 'anyo: '|| to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Calcula los importes y los parlamentarios con ms y menos gastos para el ao dado
   select max(sum(importe)), min(sum(importe)) into v_maximo, v_minimo from gasto a, parlamentario_camara b where a.id_parlamentario=b.id_parlamentario and to_number(to_char(fecha,'yyyy'))=p_anyo group by NIF;
   
   select NIF into v_NIF_maximo from gasto a, parlamentario_camara b where a.id_parlamentario=b.id_parlamentario and to_number(to_char(fecha,'yyyy'))=p_anyo group by NIF having sum(importe)=v_maximo;
   
   select NIF into v_NIF_minimo from gasto a, parlamentario_camara b where a.id_parlamentario=b.id_parlamentario and to_number(to_char(fecha,'yyyy'))=p_anyo group by NIF having sum(importe)=v_minimo;

   -- Comprueba si en la tabla estadstica 6 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA6 where anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_NIF_maximo is not null) THEN
     insert into ESTADISTICA6 values (p_anyo, v_NIF_maximo, v_NIF_minimo);
   ELSIF (v_existe_registro=1 AND v_NIF_maximo is not null) THEN
     update ESTADISTICA6 set NIF_MAS_GASTO=v_NIF_maximo, NIF_MENOS_GASTO=v_NIF_minimo where anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_NIF_maximo is null) THEN
     delete from ESTADISTICA6 where anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_6', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_6: ' || RSP;
        SP_ALTA_LOG('SP_EST_6', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_6: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_6', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_6;
/
