CREATE OR REPLACE PROCEDURE TFC.SP_EST_7 (
   p_anyo              IN ESTADISTICA7.anyo%TYPE,
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_7
   Obejtivo:               actualiza la tabla Estadsitca7
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
RSP_LOG              VARCHAR2 (2000);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
v_id_camara          INTEGER;
v_nombre             VARCHAR2 (200);
v_media_gasto        NUMBER(12,2);
v_anyo_actual        INTEGER;
v_num_parlamentarios INTEGER; 

-- Calcula la suma de todos los gastos de los parlamentarios de las distintas cmaras para el ao dado
cursor c1 is
   select a.id_camara, nombre, sum(importe) importe from gasto a, camara b where a.id_camara=b.id_camara and to_number(to_char(fecha,'yyyy'))=p_anyo group by a.id_camara, nombre;

E_CONOCIDA           EXCEPTION;   

BEGIN

   v_parametros_entrada:= 'anyo: '|| to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada es nulo
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   v_media_gasto:=0;
   -- Recorre los gastos de los parlamentarios de las distintas cmaras para el ao dado,calculando 
   -- el n de parlamentarios y el importe medio de cada una de ellas y sobre ellos el mximo

   select to_number(to_char(sysdate,'yyyy')) into v_anyo_actual from dual;

   FOR gasto_camara in c1
   LOOP
   
      IF (p_anyo=v_anyo_actual) THEN
         select count(distinct NIF) into v_num_parlamentarios from parlamentario_camara where id_camara=gasto_camara.id_camara and fecha_baja IS NULL OR to_number(to_char(fecha_baja,'yyyy'))>=p_anyo;
      ELSE
         select count(distinct NIF) into v_num_parlamentarios from parlamentario_camara where id_camara=gasto_camara.id_camara and to_number(to_char(fecha_alta,'yyyy'))<=p_anyo AND to_number(to_char(fecha_baja,'yyyy'))>=p_anyo;
      END IF;  
      IF (v_num_parlamentarios!=0) THEN
         IF (gasto_camara.importe/v_num_parlamentarios > v_media_gasto) THEN
            v_media_gasto:=gasto_camara.importe/v_num_parlamentarios;
            v_id_camara:=gasto_camara.id_camara;
            v_nombre:=gasto_camara.nombre;
         ELSE
            v_media_gasto:=gasto_camara.importe;
            v_id_camara:=gasto_camara.id_camara;
            v_nombre:=gasto_camara.nombre;
         END IF;
      END IF;
   END LOOP;

   -- Comprueba si en la tabla estadstica 7 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA7 where anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_media_gasto!=0) THEN
     insert into ESTADISTICA7 values (p_anyo, v_id_camara, v_nombre, v_media_gasto);
   ELSIF (v_existe_registro=1 AND v_media_gasto!=0) THEN
     update ESTADISTICA7 set id_camara=v_id_camara, nombre=v_nombre, media_gasto=v_media_gasto where anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_media_gasto=0) THEN
     delete from ESTADISTICA7 where anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_7', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_7: ' || RSP;
        SP_ALTA_LOG('SP_EST_7', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_7: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_7', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_7;
/
