CREATE OR REPLACE PROCEDURE TFC.SP_EST_8 (
   p_anyo              IN ESTADISTICA8.anyo%TYPE,
   RSP                 OUT VARCHAR2)
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_EST_8
   Obejtivo:               actualiza la tabla Estadsitca8
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_existe_registro    INTEGER;
v_total_gasto        NUMBER(14,2); 
E_CONOCIDA           EXCEPTION;   

BEGIN
   v_parametros_entrada:= 'anyo: '|| to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
 
   -- Calcula la suma de todos los gastos para el ao dado
   select sum(importe) into v_total_gasto from gasto where to_number(to_char(fecha,'yyyy'))=p_anyo;
   
      -- Comprueba si en la tabla estadstica 8 ya existen datos para el ao dado
   select count(*) into v_existe_registro from ESTADISTICA8 where anyo = p_anyo;
   
   IF (v_existe_registro=0 AND v_total_gasto is not null) THEN
     insert into ESTADISTICA8 values (p_anyo, v_total_gasto);
   ELSIF (v_existe_registro=1 AND v_total_gasto is not null) THEN
     update ESTADISTICA8 set gasto=v_total_gasto where anyo = p_anyo;
   ELSIF (v_existe_registro=1 AND v_total_gasto is null) THEN
     delete from ESTADISTICA8 where anyo = p_anyo;
   END IF;
    
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('SP_EST_8', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
  
   COMMIT;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_EST_8: ' || RSP;
        SP_ALTA_LOG('SP_EST_8', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:=    'ERROR SP_EST_8: ' || v_err_msg;
        SP_ALTA_LOG('SP_EST_8', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_EST_8;
/
