CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_CAMARA (
p_nombre            IN CAMARA.nombre%type,
p_id_pais           IN CAMARA.id_pais%type,
p_id_estado         IN CAMARA.id_estado%type,
p_id_region         IN CAMARA.id_region%type,
p_direccion         IN CAMARA.direccion%type,
p_tlfno             IN CAMARA.tlfno%type,
p_web               IN CAMARA.web%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_CAMARA
   Obejtivo:               Da de alta una camara
                   
   Parmetros de entrada:  p_nombre,p_id_pais,p_id_estado, p_id_region, 
                           p_direccion, p_tlfno, p_web             

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_nombre: '    || p_nombre 
                      || ' p_id_pais: '   || to_char(p_id_pais)
                      || ' p_id_region: ' || to_char(p_id_region)
                      || ' p_id_estado: ' || to_char(p_id_estado)
                      || ' p_direccion: ' || to_char(p_direccion)
                      || ' p_tlfno: '     || to_char(p_tlfno)
                      || ' p_web: '       || to_char(p_web);

   -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_nombre IS NULL) THEN 
      RSP:= 'El campo nombre no puede ser nulo' ;
      RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si ya existe una camara con ese nombre
   
   select count(*) into v_integridad from camara where nombre=p_nombre;
   IF (v_integridad!=0) THEN
      RSP:= 'Ya existe una cmara con ese nombre';
      RAISE E_CONOCIDA;
   END IF;
  
  -- Comprueba la integridad de los identificadores
   IF (p_id_pais IS NOT NULL) THEN 
      select count(*) into v_integridad from pais where id_pais=p_id_pais;
      IF (v_integridad=0) THEN
         RSP:= 'El pais no existe';
         RAISE E_CONOCIDA;
      END IF;
   END IF;

   IF (p_id_estado IS NOT NULL) THEN 
      select count(*) into v_integridad from estado where id_estado=p_id_estado;
      IF (v_integridad=0) THEN
         RSP:= 'El estado no existe';
         RAISE E_CONOCIDA;
      END IF;
   END IF;

   IF (p_id_region IS NOT NULL) THEN 
     select count(*) into v_integridad from region where id_region=p_id_region;
     IF (v_integridad=0) THEN
        RSP:= 'La regin no existe';
        RAISE E_CONOCIDA;
     END IF;
  END IF;

-- Inserta los datos de la camara:
   insert into camara (nombre, no_parlamentarios, id_pais, id_estado, id_region, direccion, tlfno, web) 
   VALUES  (p_nombre, 0, p_id_pais, p_id_estado, p_id_region, p_direccion, p_tlfno, p_web);
   RSP:='OK';
   SP_ALTA_LOG('SP_EST_2', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_CAMARA: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_CAMARA: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_CAMARA;
/
