CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_GASTO (
p_id_parlamentario  IN GASTO.id_parlamentario%type,
p_id_camara         IN GASTO.id_camara%type,
p_fecha             IN GASTO.fecha%type,
p_importe           IN GASTO.importe%type,
p_iva               IN GASTO.iva%type,
p_nif_prov          IN GASTO.nif_prov%type,
p_id_categoria      IN GASTO.id_categoria%type,
p_descripcion       IN GASTO.descripcion%type,
p_justificacion     IN GASTO.justificacion%type,
p_url               IN GASTO.url%type,
p_id_estado_gasto   IN GASTO.id_estado_gasto%type,
p_id_pago           IN GASTO.id_pago%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_GASTO
   Obejtivo:               Da de alta un gasto
                   
   Parmetros de entrada:  p_id_parlamentario, p_id_camara, p_fecha, p_importe, 
                           p_iva, p_nif_prov, p_id_categoria, p_descripcion, 
                           p_justificacion, p_url, p_id_estado_gasto, p_id_pago             

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (2000);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_id_camara          INTEGER;
v_id_partido         INTEGER;
v_tipo_gasto         VARCHAR (1);
v_NIF                VARCHAR (9);
v_anyo               INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_parlamentario:  '   || to_char(p_id_parlamentario)
                      || ' p_id_camara: '           || to_char(p_id_camara)
                      || ' p_fecha: '               || to_char(p_fecha)
                      || ' p_importe: '             || to_char(p_importe)
                      || ' p_iva: '                 || to_char(p_iva)
                      || ' p_nif_prov: '            || p_nif_prov
                      || ' p_id_categoria: '        || to_char(p_id_categoria)
                      || ' p_descripcion: '         || p_descripcion
                      || ' p_justificacion: '       || p_justificacion
                      || ' p_url: '                 || p_url
                      || ' p_id_estado_gasto: '     || to_char(p_id_estado_gasto)
                      || ' p_id_pago: '             || to_char(p_id_pago);

   IF (p_fecha >= SYSDATE) THEN 
     RSP:= 'La fecha del gasto no puede ser mayor a la fecha de hoy' ;
     RAISE E_CONOCIDA;
   END IF;


  -- Comprueba si alguno de los parmetros obligatorios es nulo  

   IF (p_fecha IS NULL) THEN 
     RSP:= 'El campo fecha no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_importe IS NULL) THEN 
     RSP:= 'El campo importe no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_iva IS NULL) THEN 
     RSP:= 'El campo iva no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_id_categoria IS NULL) THEN 
     RSP:= 'El campo categora no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_descripcion IS NULL) THEN 
     RSP:= 'El campo descripcin no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_justificacion IS NULL) THEN 
     RSP:= 'El campo justificacin no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;         
   
   IF (p_id_estado_gasto IS NULL) THEN
     RSP:= 'El campo estado del gasto no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;         
   
   IF (p_id_pago IS NULL) THEN 
     RSP:= 'El campo forma de pago no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;         

   -- Los campos p_id_parlamentario y p_id_camara no pueden estar sin informar los dos, ni estar los dos informados    
   IF (p_id_parlamentario IS NULL AND p_id_camara IS NULL) THEN 
     RSP:= 'Los campos id_parlamentario e id_camara no pueden ser ambos nulos' ;
     RAISE E_CONOCIDA;
   END IF;

   IF (p_id_parlamentario IS NOT NULL AND p_id_camara IS NOT NULL) THEN 
     RSP:= 'Los campos id_parlamentario e id_camara no pueden estar ambos informados' ;
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba la integridad de los identificadores
   IF (p_id_parlamentario IS NOT NULL) THEN
      select count(*) into v_integridad from PARLAMENTARIO_CAMARA where id_parlamentario=p_id_parlamentario;
      IF (v_integridad=0) THEN
        RSP:= 'No existe ningn parlamentario con ese id';
        RAISE E_CONOCIDA;
      END IF;  
      select id_camara into v_id_camara from PARLAMENTARIO_CAMARA where id_parlamentario=p_id_parlamentario;
      select id_partido into v_id_partido from PARLAMENTARIO a, PARLAMENTARIO_CAMARA b where a.NIF=b.NIF and id_parlamentario=p_id_parlamentario;
      ELSE
      v_id_camara:=p_id_camara;
   END IF;
   IF (p_id_camara IS NOT NULL) THEN
      select count(*) into v_integridad from camara where id_camara=p_id_camara;
      IF (v_integridad=0) THEN
         RSP:= 'No existe ninguna cmara con ese id';
         RAISE E_CONOCIDA;
      END IF;
   END IF;
   
   IF (p_id_parlamentario IS NOT NULL) THEN
      v_tipo_gasto:='P';
   ELSE
      v_tipo_gasto:='C';
   END IF;

   IF (p_id_categoria IS NOT NULL) THEN 
      select count(*) into v_integridad from CATEGORIA_GASTO where id_categoria=p_id_categoria and tipo_gasto=v_tipo_gasto;
      IF (v_integridad=0) THEN
         RSP:= 'La categora del gasto no existe para este tipo de gasto';
         RAISE E_CONOCIDA;
      END IF;
   END IF;

   select count(*) into v_integridad from ESTADO_GASTO where id_estado_gasto=p_id_estado_gasto;
   IF (v_integridad=0) THEN
      RSP:= 'El estado de gasto no existe';
      RAISE E_CONOCIDA;
   END IF;

   select count(*) into v_integridad from FORMA_PAGO where id_pago=p_id_pago;
   IF (v_integridad=0) THEN
      RSP:= 'La forma de pago no existe';
      RAISE E_CONOCIDA;
   END IF;

-- Inserta los datos de la camara:
   insert into GASTO (id_parlamentario, id_camara, id_partido, fecha, importe, iva, nif_prov, id_categoria, 
                      descripcion, justificacion, url, id_estado_gasto, id_pago) 
   VALUES  (p_id_parlamentario, v_id_camara, v_id_partido, p_fecha, p_importe, p_iva, p_nif_prov, p_id_categoria, 
            p_descripcion, p_justificacion, p_url, p_id_estado_gasto, p_id_pago);
            
   COMMIT;
   -- Actualizamos todas las estadsticas

   v_anyo:=to_char(p_fecha,'yyyy');

   SP_EST_1(v_id_camara,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 1: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;         
   
   IF (p_id_camara IS NULL) THEN   
      SP_EST_2(v_anyo,v_id_camara,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF; 
   
   SP_EST_3(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 3: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (p_id_camara IS NULL) THEN
      SELECT NIF INTO v_nif FROM PARLAMENTARIO_CAMARA WHERE id_parlamentario=p_id_parlamentario;   
      SP_EST_4(v_nif,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF; 
   
   IF (p_id_camara IS NULL) THEN   
      SP_EST_5(v_anyo,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF; 

   IF (p_id_camara IS NULL) THEN   
      SP_EST_6(v_anyo,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF; 

   SP_EST_7(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 7: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;

   SP_EST_8(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 8: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;

   RSP:='OK';
   SP_ALTA_LOG('SP_ALTA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_GASTO: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_GASTO: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
END SP_ALTA_GASTO;
/
