CREATE OR REPLACE PROCEDURE TFC.SP_ALTA_PARLAM (
p_NIF               IN PARLAMENTARIO.NIF%type,
p_nombre_apellidos  IN PARLAMENTARIO.nombre_apellidos%type,
p_id_partido        IN PARLAMENTARIO.id_partido%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_ALTA_PARLAM
   Obejtivo:               Da de alta un parlamentario
                   
   Parmetros de entrada:  p_NIF, p_nombre_apellidos, p_id_partido                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_NIF: '               || p_NIF 
                      || ' p_nombre_apellidos: '  || p_nombre_apellidos
                      || ' p_id_partido: '        || to_char(p_id_partido);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_NIF IS NULL) THEN 
     RSP:= 'El campo NIF no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   IF (p_nombre_apellidos IS NULL) THEN 
     RSP:= 'El campo nombre_apellidos no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   IF (p_id_partido IS NULL) THEN 
     RSP:= 'El campo id_partido no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe un parlamentario con ese NIF
   select count(*) into v_integridad from PARLAMENTARIO where NIF=p_NIF;
   IF (v_integridad!=0) THEN
     RSP:= 'Ya existe un parlamentario con ese NIF';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si existe un partido con ese id
   select count(*) into v_integridad from PARTIDO where id_partido=p_id_partido;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ningn partido con ese id';
     RAISE E_CONOCIDA;
   END IF;

   INSERT INTO PARLAMENTARIO (NIF, Nombre_Apellidos, Id_Partido) VALUES (p_NIF, p_Nombre_Apellidos, p_Id_Partido);

   RSP:='OK';
   SP_ALTA_LOG('SP_ALTA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_ALTA_PARLAM: ' || RSP;
        SP_ALTA_LOG('SP_ALTA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_ALTA_PARLAM: ' || v_err_msg;
        SP_ALTA_LOG('SP_ALTA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_ALTA_PARLAM;
/
