CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_CAMARA (
p_id_camara         IN CAMARA.id_camara%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_CAMARA
   Obejtivo:               Da de alta una camara
                   
   Parmetros de entrada:  p_id_camara
                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_camara: ' || to_char(p_id_camara);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_camara IS NULL) THEN 
     RSP:= 'El campo id_camara no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe una camara con ese id
   select count(*) into v_integridad from camara where id_camara=p_id_camara;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si hay parlamentarios asociados a esa cmara
   select count(*) into v_integridad from parlamentario_camara where id_camara=p_id_camara;
   IF (v_integridad!=0) THEN
     RSP:= 'Hay parlamentarios asociados a esa cmara';
     RAISE E_CONOCIDA;
   END IF;
  
   -- Comprueba si hay gastos asociados a esa cmara
   select count(*) into v_integridad from gasto where id_camara=p_id_camara;
   IF (v_integridad!=0) THEN
     RSP:= 'Hay gastos asociados a esa cmara';
     RAISE E_CONOCIDA;
   END IF;

-- Borra los datos de la camara:
   delete from camara where id_camara=p_id_camara;
   RSP:='OK';
   SP_ALTA_LOG('SP_BAJA_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_CAMARA: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_CAMARA: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_CAMARA;
/
