CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_GASTO (
p_id_gasto          IN GASTO.id_gasto%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_GASTO
   Obejtivo:               Da de baja un gasto
                   
   Parmetros de entrada:  p_id_gasto             

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (2000);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_id_parlamentario   INTEGER;
v_id_camara            INTEGER;
v_NIF                VARCHAR (9);
v_anyo               INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_gasto:  '   || to_char(p_id_gasto);

   -- Comprueba si alguno de los parmetros obligatorios es nulo  

    IF (p_id_gasto IS NULL) THEN 
      RSP:= 'El campo identificador del gasto no puede ser nulo' ;
      RAISE E_CONOCIDA;
    END IF;

   -- Comprueba la integridad de los identificadores
    select count(*) into v_integridad from GASTO where id_gasto=p_id_gasto;
    IF (v_integridad=0) THEN
       RSP:= 'No existe ningn gasto con ese id';
       RAISE E_CONOCIDA;
    END IF;  

   SELECT to_char(fecha,'yyyy'), id_parlamentario, id_camara INTO v_anyo, v_id_parlamentario, v_id_camara FROM GASTO WHERE id_gasto=p_id_gasto;

   -- Inserta los datos de la camara:
   DELETE FROM GASTO WHERE id_gasto=p_id_gasto;
   COMMIT;
   -- Actualizamos todas las estadsticas  

   SP_EST_1(v_id_camara,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 1: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;         
   
   SP_EST_2(v_anyo,v_id_camara,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 2: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;
   
   SP_EST_3(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 3: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;
   
   IF (v_id_parlamentario IS NOT NULL) THEN
      SELECT NIF INTO v_nif FROM PARLAMENTARIO_CAMARA WHERE id_parlamentario=v_id_parlamentario;   
      SP_EST_4(v_nif,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 4: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;
   END IF; 
   
   SP_EST_5(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 5: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;
   
   SP_EST_6(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 6: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;

   SP_EST_7(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 7: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;

   SP_EST_8(v_anyo,RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='Error en el clculo de la estadstica 8: '|| RSP_LOG;
      RAISE E_CONOCIDA;
   END IF;
   
   RSP:='OK';
   SP_ALTA_LOG('SP_BAJA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_GASTO: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_GASTO: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_GASTO', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_GASTO;
/
