CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_P_CAM (
p_id_parlamentario               IN PARLAMENTARIO_CAMARA.id_parlamentario%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_P_CAM
   Obejtivo:               Da de baja un parlamentario en una cmara
                   
   Parmetros de entrada:  p_id_parlamentario
                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_id_camara          INTEGER;
v_anyo_alta          INTEGER;
v_anyo_baja          INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_parlamentario: ' || p_id_parlamentario;

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_id_parlamentario IS NULL) THEN 
     RSP:= 'El campo identificador no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe un parlamentario con ese id_parlamentario
   select count(*) into v_integridad from PARLAMENTARIO_CAMARA where id_parlamentario=p_id_parlamentario;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ningn parlamentario con ese identificador';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si el parlamentario tiene gastos asociados
   select count(*) into v_integridad from GASTO a where id_parlamentario=p_id_parlamentario;
   IF (v_integridad!=0) THEN
     RSP:= 'El parlamentario tiene gastos asociados';
     RAISE E_CONOCIDA;
   END IF;
  
   -- Comprobamos que en parlamentario no est dado de alta en ninguna cmara 
   -- en el momento actual, en caso contrario, restamos 1 al n de parlamentarios de dicha cmara
   select count(*) into v_integridad from PARLAMENTARIO_CAMARA WHERE id_parlamentario=p_id_parlamentario and fecha_baja is null;
   IF (v_integridad!=0) THEN
      select id_camara into v_id_camara from PARLAMENTARIO_CAMARA WHERE id_parlamentario=p_id_parlamentario and fecha_baja is null;
      IF (v_id_camara is not null) THEN
         UPDATE CAMARA set no_parlamentarios=no_parlamentarios-1 where id_camara=v_id_camara;
      END IF;
   END IF;
   select to_number(to_char(fecha_alta,'yyyy')), to_number(to_char(nvl(fecha_baja,sysdate),'yyyy')) 
   into v_anyo_alta,v_anyo_baja from parlamentario_camara where id_parlamentario=p_id_parlamentario;
   

   DELETE FROM PARLAMENTARIO_CAMARA WHERE id_parlamentario=p_id_parlamentario;
   COMMIT;
   
     -- Actualiza las estadsitcas 5 y 7, que se calculan en funcin del nmero de parlamentarios por cmara. 
   FOR anyo IN v_anyo_alta..v_anyo_baja
   LOOP
      SP_EST_5(anyo,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 5: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;         
      SP_EST_7(anyo,RSP_LOG);
      IF (RSP_LOG!='OK') THEN
         RSP:='Error en el clculo de la estadstica 7: '|| RSP_LOG;
         RAISE E_CONOCIDA;
      END IF;         
   END LOOP;

              
   RSP:='OK';
   SP_ALTA_LOG('SP_BAJA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_P_CAM: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_P_CAM: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_P_CAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_BAJA_P_CAM;
/
