CREATE OR REPLACE PROCEDURE TFC.SP_BAJA_PARLAM (
p_NIF               IN PARLAMENTARIO.NIF%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_BAJA_PARLAM
   Obejtivo:               Da de baja un parlamentario
                   
   Parmetros de entrada:  p_NIF
                                        

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
v_id_camara          INTEGER;
v_anyo_alta          INTEGER;
v_anyo_baja          INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_NIF: ' || p_NIF;

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_NIF IS NULL) THEN 
     RSP:= 'El campo NIF no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si existe un parlamentario con ese NIF
   select count(*) into v_integridad from PARLAMENTARIO where NIF=p_NIF;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ningn parlamentario con ese NIF';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si el parlamentario tiene gastos asociados
   select count(*) into v_integridad from PARLAMENTARIO_CAMARA where NIF=p_NIF;
   IF (v_integridad!=0) THEN
     RSP:= 'El parlamentario est o ha estado dado de alta en alguna cmara';
     RAISE E_CONOCIDA;
   END IF;

   DELETE FROM PARLAMENTARIO WHERE NIF=p_NIF;
              
   RSP:='OK';
   SP_ALTA_LOG('SP_BAJA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_BAJA_PARLAM: ' || RSP;
        SP_ALTA_LOG('SP_BAJA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);


   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_BAJA_PARLAM: ' || v_err_msg;
        SP_ALTA_LOG('SP_BAJA_PARLAM', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);


END SP_BAJA_PARLAM;
/
