CREATE OR REPLACE PROCEDURE TFC.SP_MOD_CAMARA (
p_id_camara         IN CAMARA.id_camara%type,
p_nombre            IN CAMARA.nombre%type,
p_id_pais           IN CAMARA.id_pais%type,
p_id_estado         IN CAMARA.id_estado%type,
p_id_region         IN CAMARA.id_region%type,
p_direccion         IN CAMARA.direccion%type,
p_tlfno             IN CAMARA.tlfno%type,
p_web               IN CAMARA.web%type,
RSP                 OUT VARCHAR2)

IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 SP_MOD_CAMARA
   Obejtivo:               Modifica los datos de una camara
                   
   Parmetros de entrada:  p_nombre,p_id_pais,p_id_estado, p_id_region, 
                           p_direccion, p_tlfno, p_web             

   Resultado:             'OK' si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
******************************************************************************/
v_parametros_entrada VARCHAR2 (200);
RSP_LOG              VARCHAR2 (2000);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
E_CONOCIDA           EXCEPTION;

BEGIN
   v_parametros_entrada:= 'p_id_camara: '   || to_char(p_id_camara)
                      || ' p_nombre: '    || p_nombre 
                      || ' p_id_pais: '   || to_char(p_id_pais)
                      || ' p_id_region: ' || to_char(p_id_region)
                      || ' p_id_estado: ' || to_char(p_id_estado)
                      || ' p_direccion: ' || to_char(p_direccion)
                      || ' p_tlfno: '     || to_char(p_tlfno)
                      || ' p_web: '       || to_char(p_web);

  -- Comprueba si alguno de los parmetros obligatorios es nulo
   IF (p_nombre IS NULL) THEN 
     RSP:= 'El campo nombre no puede ser nulo' ;
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si ya existe una camara con ese nombre
   
   select count(*) into v_integridad from camara where id_camara=p_id_camara;
   IF (v_integridad=0) THEN
     RSP:= 'No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   select count(*) into v_integridad from camara where nombre=p_nombre and id_camara!=p_id_camara;
   IF (v_integridad!=0) THEN
     RSP:= 'Ya existe una cmara con ese nombre';
     RAISE E_CONOCIDA;
   END IF;
  
  -- Comprueba la integridad de los identificadores
  IF (p_id_pais IS NOT NULL) THEN 
     select count(*) into v_integridad from pais where id_pais=p_id_pais;
     IF (v_integridad=0) THEN
        RSP:= 'El pais no existe';
        RAISE E_CONOCIDA;
     END IF;
  END IF;

  IF (p_id_estado IS NOT NULL) THEN 
  select count(*) into v_integridad from estado where id_estado=p_id_estado;
     IF (v_integridad=0) THEN
        RSP:= 'El estado no existe';
        RAISE E_CONOCIDA;
     END IF;
  END IF;

  IF (p_id_region IS NOT NULL) THEN 
  select count(*) into v_integridad from region where id_region=p_id_region;
     IF (v_integridad=0) THEN
        RSP:= 'La regin no existe';
        RAISE E_CONOCIDA;
     END IF;
  END IF;

-- Inserta los datos de la camara:
   update camara set nombre=p_nombre, id_pais=p_id_pais, id_estado=p_id_estado, 
   id_region=p_id_region, direccion=p_direccion, tlfno=p_tlfno, web=p_web where id_camara=p_id_camara;
   RSP:='OK';
   SP_ALTA_LOG('SP_MODIFICACION_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;

   COMMIT;

EXCEPTION       
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR SP_MOD_CAMARA: ' || RSP;
        SP_ALTA_LOG('SP_MOD_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:= 'ERROR SP_MOD_CAMARA: ' || v_err_msg;
        SP_ALTA_LOG('SP_MOD_CAMARA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);

END SP_MOD_CAMARA;
/
