/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R6a AS OBJECT (
NIF                VARCHAR2(9),
NOMBRE_APELLIDOS   VARCHAR2(200),
ID_CAMARA          INTEGER,
NOMBRE_CAMARA      VARCHAR2(200),
NOMBRE_PARTIDO     VARCHAR2(200),
FECHA              DATE,
IMPORTE            NUMBER(12,2),
IVA                INTEGER,
NIF_PROV           VARCHAR2(9),
CATEGORIA          VARCHAR2(200),
DESCRIPCION        VARCHAR2(200),
JUSTIFICACION      VARCHAR2(200),
URL                VARCHAR2(200),
DESC_ESTADO_GASTO  VARCHAR2(200),
DESC_FORMA_PAGO    VARCHAR2(200)
);
/
   
 
/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R6a AS TABLE OF TY_R6a;             
/

/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION ConsultaA (
p_id_camara   IN GASTO.id_camara%type,
p_anyo        IN INTEGER,
p_nif         IN PARLAMENTARIO.NIF%type
)
RETURN TA_R6A
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                ConsultaA
   Obejtivo:              Dado un parlamento , un ao y un parlamentario.:
                          el listado de todos los gastos asociados al parlamentario en ese ao .
                   
   Parmetros de entrada:  p_id_camara, p_anyo, p_NIF            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_diferencia         NUMBER(14,2); 
v_integridad         INTEGER;
v_existe_registro    INTEGER;
i                    INTEGER:=1;
resultado            TA_R6a:=TA_R6a();
nif                  VARCHAR2(9);
nombre_apellidos     VARCHAR2(200);
id_camara            INTEGER;
nombre_camara        VARCHAR2(200);
nombre_partido       VARCHAR2(200);
fecha                DATE;
importe              NUMBER(12,2);
iva                  INTEGER;
nif_prov             VARCHAR2(9);
categoria            VARCHAR2(200);
descripcion          VARCHAR2(200);
justificacion        VARCHAR2(200);
url                  VARCHAR2(200);
desc_estado_gasto    VARCHAR2(200);
desc_forma_pago      VARCHAR2(200);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_id_camara: '|| to_char(p_id_camara)
                      || 'p_anyo: '     || to_char(p_anyo)
                      || 'p_nif: '     || p_nif;

  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_id_camara IS NULL) THEN 
     RSP:='El campo id_camara no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   IF (p_nif IS NULL) THEN 
     RSP:='El campo NIF no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si la cmara existe
   select count(*) into v_integridad from camara where id_camara = p_id_camara;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   -- Comprueba si parlamentario existe
   select count(*) into v_integridad from parlamentario where nif = p_nif;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ningn parlamentario con ese nif';
     RAISE E_CONOCIDA;
   END IF;
   
   OPEN CUR FOR 
       SELECT b.nif, d.nombre_apellidos, a.id_camara, c.nombre nombre_camara , e.nombre nombre_partido, a.fecha,
       a.importe, a.iva, a.nif_prov, f.descripcion categoria, a.descripcion, a.justificacion, a.url, 
       g.descripcion desc_estado_gasto, h.DESCRIPCION desc_forma_pago
       FROM GASTO a, PARLAMENTARIO_CAMARA b, CAMARA c, PARLAMENTARIO d, PARTIDO e, CATEGORIA_GASTO f,
       ESTADO_GASTO g, FORMA_PAGO h WHERE a.id_parlamentario=b.id_parlamentario and a.id_camara=b.id_camara 
       and d.NIF=b.NIF and a.id_camara=c.id_camara and b.NIF=p_NIF and c.id_camara=p_id_camara 
       and e.id_partido=a.id_partido and to_number(to_char(fecha,'yyyy'))=p_anyo and a.id_categoria=f.id_categoria
       and a.id_estado_gasto=g.id_estado_gasto and a.id_pago=h.id_pago order by importe desc;
   LOOP
   
     FETCH CUR INTO nif, nombre_apellidos, id_camara, nombre_camara , nombre_partido, fecha, importe, 
       iva, nif_prov, categoria, descripcion, justificacion, url, desc_estado_gasto, desc_forma_pago;
     EXIT WHEN CUR%NOTFOUND;
       resultado.extend;
       resultado(i):= TY_R6a(nif, nombre_apellidos, id_camara, nombre_camara , nombre_partido, fecha, importe,
                iva, nif_prov, categoria, descripcion, justificacion, url, desc_estado_gasto, desc_forma_pago);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaA: ' || RSP;
        SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaA: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null; 

END ConsultaA;
/



  