CREATE OR REPLACE TYPE TY_R6b AS OBJECT (
NOMBRE_APELLIDOS   VARCHAR2(200),
NOMBRE_PARTIDO     VARCHAR2(200),
GASTO_TOTAL        NUMBER(12,2)
);
/
   
 
CREATE OR REPLACE TYPE TA_R6b AS TABLE OF TY_R6b;             
/

CREATE OR REPLACE FUNCTION ConsultaB (
p_id_camara   IN GASTO.id_camara%type,
p_anyo        IN INTEGER
)
RETURN TA_R6b
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                ConsultaB
   Obejtivo:              Dado un parlamento, y un ao, el listado de los 
                          parlamentarios con el gasto directo que ha hecho cada uno .
                   
   Parmetros de entrada:  p_id_camara, p_anyo            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
i                    INTEGER:=1;
resultado            TA_R6b:=TA_R6b();
nombre_apellidos     VARCHAR2(200);
nombre_partido       VARCHAR2(200);
gasto_total              NUMBER(12,2);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_id_camara: '|| to_char(p_id_camara)
                      || 'p_anyo: '     || to_char(p_anyo);

  -- Comprueba si el parmetro de entrada sea nulo
   IF (p_id_camara IS NULL) THEN 
     RSP:='El campo id_camara no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;
   
   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si la cmara existe
   select count(*) into v_integridad from camara where id_camara = p_id_camara;
   
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   OPEN CUR FOR 
       SELECT d.nombre_apellidos, e.nombre nombre_partido, sum(a.importe) gasto_total
       FROM GASTO a, PARLAMENTARIO_CAMARA b, CAMARA c, PARLAMENTARIO d, PARTIDO e 
       WHERE a.id_parlamentario=b.id_parlamentario and a.id_camara=b.id_camara 
       and d.NIF=b.NIF and a.id_camara=c.id_camara and c.id_camara=p_id_camara 
       and e.id_partido=a.id_partido and to_number(to_char(fecha,'yyyy'))=p_anyo 
       group by nombre_apellidos, e.nombre order by sum(a.importe);
   LOOP
   
     FETCH CUR INTO nombre_apellidos, nombre_partido, gasto_total;
     EXIT WHEN CUR%NOTFOUND;
       resultado.extend;
       resultado(i):= TY_R6b(nombre_apellidos, nombre_partido, gasto_total);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   DBMS_OUTPUT.PUT_LINE(RSP);
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaA: ' || RSP;
        SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;
        DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaA: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaA', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN null; 

END ConsultaB;
/

