CREATE OR REPLACE TYPE TY_R6c AS OBJECT (
NOMBRE_CAMARA       VARCHAR2(200),
NO_PARLAMENTARIOS   INTEGER,
GASTO_MEDIO         NUMBER(14,2),
SUMA_GASTO_PARLAM   NUMBER(14,2),
SUMA_GASTO_CAMARA   NUMBER(14,2)
);
/
   
CREATE OR REPLACE TYPE TA_R6c AS TABLE OF TY_R6c;             
/

CREATE OR REPLACE FUNCTION ConsultaC (
p_anyo        IN INTEGER
)
RETURN TA_R6c
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                ConsultaC
   Obejtivo:              Dado un ao: listado del gasto total del todos los parlamentos 
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
i                    INTEGER:=1;
resultado            TA_R6c:=TA_R6c();
v_ID_CAMARA            INTEGER; 
NOMBRE_CAMARA        VARCHAR2(200);
NO_PARLAMENTARIOS    INTEGER;
GASTO_MEDIO          NUMBER(14,2);
SUMA_GASTO_PARLAM    NUMBER(14,2);
SUMA_GASTO_CAMARA    NUMBER(14,2);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_anyo: '     || to_char(p_anyo);

  -- Comprueba si el parmetro de entrada sea nulo

   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   OPEN CUR FOR 
       SELECT id_camara v_id_camara
       FROM GASTO a WHERE to_number(to_char(fecha,'yyyy'))=p_anyo group by id_camara;
   LOOP
      FETCH CUR INTO v_id_camara;
      EXIT WHEN CUR%NOTFOUND;
         SELECT nombre into nombre_camara FROM CAMARA where id_camara=v_id_camara;
         IF (p_anyo=to_number(to_char(sysdate,'yyyy'))) THEN
            select count(distinct NIF) into no_parlamentarios from parlamentario_camara 
            where id_camara=v_id_camara and (to_number(to_char(fecha_baja,'yyyy'))=p_anyo OR fecha_baja IS NULL);
         ELSE
            select count(distinct NIF) into no_parlamentarios from parlamentario_camara 
            where id_camara=v_id_camara and to_number(to_char(fecha_alta,'yyyy'))<=p_anyo and to_number(to_char(fecha_baja,'yyyy'))>=p_anyo;
         END IF;  

         select sum(importe) into suma_gasto_parlam FROM GASTO WHERE id_camara=v_id_camara and  
         to_number(to_char(fecha,'yyyy'))=p_anyo and id_parlamentario IS NOT NULL; 
         select sum(importe) into suma_gasto_camara FROM GASTO WHERE id_camara=v_id_camara and  
         to_number(to_char(fecha,'yyyy'))=p_anyo and id_parlamentario IS NULL;
       resultado.extend;     
       resultado(i):= TY_R6c(nombre_camara, no_parlamentarios, suma_gasto_parlam/to_number(no_parlamentarios), suma_gasto_parlam, suma_gasto_camara);
       i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   DBMS_OUTPUT.PUT_LINE(RSP);
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaC: ' || RSP;
        SP_ALTA_LOG('ConsultaC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;
        DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaC: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaC', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN null; 

END ConsultaC;
/

