/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R6d AS OBJECT (
ID_CATEGORIA         INTEGER,
CATEGORIA            VARCHAR2(200),
SUMA_GASTO           NUMBER(14,2)
);
/
   
/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R6d AS TABLE OF TY_R6d;             
/

/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION ConsultaD (
p_anyo        IN INTEGER,
p_id_camara   IN INTEGER
)
RETURN TA_R6d
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 ConsultaD
   Obejtivo:               Dado un ao y un parlamento: el listado del gasto segmentada
                           por categoras de gasto 
                   
   Parmetros de entrada:  p_anyo, p_id_camara            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
i                    INTEGER:=1;
resultado            TA_R6d:=TA_R6d();
ID_CATEGORIA         INTEGER; 
CATEGORIA            VARCHAR2(200);
SUMA_GASTO           NUMBER(14,2);

type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_anyo: '     || to_char(p_anyo)
                      || 'p_id_camara: '|| to_char(p_id_camara);
  -- Comprueba si el parmetro de entrada sea nulo

   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   IF (p_id_camara IS NULL) THEN 
     RSP:='El campo id_camara no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si la cmara existe
   select count(*) into v_integridad from camara where id_camara = p_id_camara;  
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   OPEN CUR FOR 
       SELECT a.id_categoria, b.descripcion categoria, sum(importe) suma_gasto
       FROM GASTO a, CATEGORIA_GASTO b WHERE to_number(to_char(fecha,'yyyy'))=p_anyo 
       and a.id_camara=p_id_camara and a.id_categoria=b.id_categoria
       group by a.id_categoria, b.descripcion;
   LOOP
   
      FETCH CUR INTO id_categoria, categoria, suma_gasto;
      EXIT WHEN CUR%NOTFOUND;
         resultado.extend;
         resultado(i):= TY_R6d(id_categoria, categoria, suma_gasto);
         i:= i+1;
   END LOOP;  
   CLOSE CUR;  
   
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaD', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   DBMS_OUTPUT.PUT_LINE(RSP);
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaD: ' || RSP;
        SP_ALTA_LOG('ConsultaD', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;
        DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaD: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaD', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN null; 

END ConsultaD;
/

SELECT * FROM TABLE(ConsultaD (2010,3));