/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TY_R6e AS OBJECT (
NOMBRE_APELLIDOS     VARCHAR2(200),
PARTIDO_POLITICO     VARCHAR2(200),
SUMA_GASTO           NUMBER(14,2),
PORCENTAJE           NUMBER(14,2)
);
/
/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE TYPE TA_R6e AS TABLE OF TY_R6e;             
/


/*<TOAD_FILE_CHUNK>*/
CREATE OR REPLACE FUNCTION ConsultaE (
p_anyo        IN INTEGER,
p_id_camara   IN INTEGER
)
RETURN TA_R6e
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 ConsultaE
   Obejtivo:               Dado un ao y un parlamento: el listado de parlamentarios 
                           que superan el gasto medio de los todos los parlamentarios 
                           de aquel parlamento durante el ao indicado 
                   
   Parmetros de entrada:  p_anyo, p_id_camara            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
v_integridad         INTEGER;
i                    INTEGER:=1;
resultado            TA_R6e:=TA_R6e();
NOMBRE_APELLIDOS     VARCHAR2(200);
PARTIDO_POLITICO     VARCHAR2(200);
SUMA_GASTO           NUMBER(14,2);
PORCENTAJE           NUMBER(5,2);
suma_total_gasto     NUMBER(14,2);
no_parlamentarios    INTEGER;
type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_anyo: '     || to_char(p_anyo)
                      || 'p_id_camara: '|| to_char(p_id_camara);
  -- Comprueba si el parmetro de entrada sea nulo

   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   IF (p_id_camara IS NULL) THEN 
     RSP:='El campo id_camara no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   -- Comprueba si la cmara existe
   select count(*) into v_integridad from camara where id_camara = p_id_camara;  
   IF (v_integridad=0) THEN
     RSP:='No existe ninguna cmara con ese id';
     RAISE E_CONOCIDA;
   END IF;
   
   OPEN CUR FOR 
       SELECT nombre_apellidos, nombre partido_politico, sum(importe) suma_gasto
       FROM GASTO a, PARLAMENTARIO_CAMARA b, PARLAMENTARIO c, PARTIDO d 
       WHERE to_number(to_char(fecha,'yyyy'))=p_anyo and a.id_camara=p_id_camara 
       AND a.id_camara=b.id_camara and b.NIF=C.NIF AND c.id_partido=d.id_partido
       AND a.id_parlamentario=b.id_parlamentario group by nombre_apellidos, nombre;
   LOOP
   
      FETCH CUR INTO nombre_apellidos, partido_politico, suma_gasto;
      EXIT WHEN CUR%NOTFOUND;
         resultado.extend;
         IF (p_anyo=to_number(to_char(sysdate,'yyyy'))) THEN
            select count(distinct NIF) into no_parlamentarios from parlamentario_camara 
            where id_camara=p_id_camara and (to_number(to_char(fecha_baja,'yyyy'))=p_anyo OR fecha_baja IS NULL);
         ELSE
            select count(distinct NIF) into no_parlamentarios from parlamentario_camara 
            where id_camara=p_id_camara and to_number(to_char(fecha_alta,'yyyy'))<=p_anyo and to_number(to_char(fecha_baja,'yyyy'))>=p_anyo;
         END IF;  

         select sum(importe) into suma_total_gasto FROM GASTO WHERE id_camara=p_id_camara and  
         to_number(to_char(fecha,'yyyy'))=p_anyo and id_parlamentario IS NOT NULL; 

         IF (suma_total_gasto/no_parlamentarios < suma_gasto) THEN
            porcentaje:=((suma_gasto - (suma_total_gasto/no_parlamentarios))/(suma_total_gasto/no_parlamentarios))*100;
            resultado(i):= TY_R6e(nombre_apellidos, partido_politico, suma_gasto, porcentaje);
            i:= i+1;
         END IF;
   END LOOP;  
   CLOSE CUR;  
   
  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   DBMS_OUTPUT.PUT_LINE(RSP);
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaE: ' || RSP;
        SP_ALTA_LOG('ConsultaE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;
        DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaE: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaE', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN null; 

END ConsultaE;

/

