CREATE OR REPLACE TYPE TY_R6f AS OBJECT (
ID_CAMARA            INTEGER,
NOMBRE_CAMARA        VARCHAR2(200),
NO_PARLAMENTARIOS    INTEGER,
GASTO_PDTE_APROB     NUMBER(14,2),
GASTO_APROB          NUMBER(14,2),
GASTO_PDTE_ABONO     NUMBER(14,2),
GASTO_ABONADO        NUMBER(14,2)
);
/
CREATE OR REPLACE TYPE TA_R6f AS TABLE OF TY_R6f;             
/

CREATE OR REPLACE FUNCTION ConsultaF (
p_anyo        IN INTEGER
)
RETURN TA_R6f
IS
PRAGMA AUTONOMOUS_TRANSACTION;
/******************************************************************************
   Nombre:                 ConsultaF
   Obejtivo:               Dado un ao: listado de todos los parlamentos con su estado contable 
                   
   Parmetros de entrada:  p_anyo            

   Resultado:             'OK' + resultado de la consulta si la ejecucin termina correctamente
                          'ERROR: tipo de error' si ha fallado
                          
******************************************************************************/
RSP                  VARCHAR2 (200);
RSP_LOG              VARCHAR2 (200);
v_parametros_entrada VARCHAR2 (200);
v_err_msg            VARCHAR2 (200);
i                    INTEGER:=1;
resultado            TA_R6f:=TA_R6f();
v_ID_CAMARA          INTEGER;
NOMBRE_CAMARA        VARCHAR2(200);
NO_PARLAMENTARIOS    INTEGER;
GASTO_PDTE_APROB     NUMBER(14,2);
GASTO_APROB          NUMBER(14,2);
GASTO_PDTE_ABONO     NUMBER(14,2);
GASTO_ABONADO        NUMBER(14,2);
type r_cursor IS REF CURSOR;
CUR r_cursor;

E_CONOCIDA           EXCEPTION;

BEGIN 

   v_parametros_entrada:='p_anyo: '     || to_char(p_anyo);
   
  -- Comprueba si el parmetro de entrada sea nulo

   IF (p_anyo IS NULL) THEN 
     RSP:='El campo ao no puede ser nulo';
     RAISE E_CONOCIDA;
   END IF;

   OPEN CUR FOR 
       SELECT id_camara FROM GASTO a WHERE to_number(to_char(fecha,'yyyy'))=p_anyo GROUP BY id_camara;
   LOOP
   
      FETCH CUR INTO v_id_camara;
      EXIT WHEN CUR%NOTFOUND;
         SELECT nombre, no_parlamentarios into nombre_camara, no_parlamentarios from camara where id_camara=v_id_camara;
         SELECT nvl(sum(importe),0) INTO gasto_pdte_aprob FROM GASTO WHERE id_camara=v_id_camara AND id_estado_gasto=1 AND to_number(to_char(fecha,'yyyy'))=p_anyo;
         SELECT nvl(sum(importe),0) INTO gasto_aprob FROM GASTO WHERE id_camara=v_id_camara AND id_estado_gasto=2 AND to_number(to_char(fecha,'yyyy'))=p_anyo;
         SELECT nvl(sum(importe),0) INTO gasto_pdte_abono FROM GASTO WHERE id_camara=v_id_camara AND id_estado_gasto=3 AND to_number(to_char(fecha,'yyyy'))=p_anyo;
         SELECT nvl(sum(importe),0) INTO gasto_abonado FROM GASTO WHERE id_camara=v_id_camara AND id_estado_gasto=4 AND to_number(to_char(fecha,'yyyy'))=p_anyo;
         
         resultado.extend;
         resultado(i):= TY_R6F(v_id_camara, nombre_camara, no_parlamentarios, gasto_pdte_aprob, gasto_aprob, gasto_pdte_abono, gasto_abonado);
         i:= i+1;
   END LOOP;  
   CLOSE CUR;  

  -- terminado ok
   RSP:= 'OK';

  -- llamada al procedimiento que carga la tabla de log
   SP_ALTA_LOG('ConsultaF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
   IF (RSP_LOG!='OK') THEN
      RSP:='No ha sido posible dar de alta el log: '|| RSP_LOG;
      ROLLBACK;
   END IF;
   DBMS_OUTPUT.PUT_LINE(RSP);
   
   RETURN resultado;

EXCEPTION
   WHEN E_CONOCIDA THEN
        ROLLBACK;
        RSP:= 'ERROR ConsultaF: ' || RSP;
        SP_ALTA_LOG('ConsultaF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        RETURN null;
        DBMS_OUTPUT.PUT_LINE(RSP);

   WHEN OTHERS THEN
        ROLLBACK;
        v_err_msg := SUBSTR (SQLERRM, 1, 200);
        RSP:='ERROR ConsultaF: ' || v_err_msg;
        SP_ALTA_LOG('ConsultaF', v_parametros_entrada, 'RSP: '||RSP, RSP_LOG);
        DBMS_OUTPUT.PUT_LINE(RSP);
        RETURN null; 

END ConsultaF;

/

SELECT * FROM TABLE (CONSULTAF(2010))